#! /usr/bin/env python

# pymuseekd - Python tools for museekd
#
# Copyright (C) 2003-2004 Hyriand <hyriand@thegraveyard.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from museek import messages, driver
import sys, time, os

class Mulog(driver.Driver):
	def __init__(self, logger):
		driver.Driver.__init__(self)
		self.logger = logger
		
	def cb_disconnected(self):
		self.logger.fatal("disconnected from museekd, bailing out")
	
	def cb_login_error(self, error):
		self.logger.fatal("login error trying to connect to museekd:", error)
		try:
			self.socket.close();
		except Exception, e:
			pass
		self.socket = None
	
	def cb_server_state(self, state, username):
		if not state:
			self.logger.status("not connected to SoulSeek")
		else:
			self.logger.status("connected to SoulSeek, your username:", username)
	
	def cb_private_message(self, timestamp, user, message):
		self.logger.private(timestamp, user, message)
	
	def cb_room_said(self, room, user, message):
		self.logger.room(room, user, message)


class Logger:
	def __init__(self, path):
		path = os.path.expanduser(path)
		if not os.path.isdir(path):
			try:
				os.makedirs(path)
			except Exception, e:
				sys.stderr.write("couldn't create log path %s\n" % path)
				sys.stderr.write(e + "\n")
				sys.exit(1)
		self.path = path
		
	def write(self, path, message, timestamp = None):
		if timestamp is None:
			timestamp = time.time()
		timestamp = time.strftime("%d %b %Y %H:%M:%S", time.localtime(timestamp))
		
		path = os.path.join(self.path, path)
		dir = os.path.split(path)[0]
		if not os.path.isdir(dir):
			os.makedirs(dir)
		f = open(path, "a")
		f.write("%s %s\n" % (timestamp, message))
		f.close()
		
	def fatal(self, *args):
		m = " ".join(args)
		sys.stderr.write(m)
		self.write("syslog", m)
		sys.exit(1)
	
	def status(self, *args):
		self.write("syslog", " ".join(args))
	
	def private(self, timestamp, user, message):
		if message[:4] == "/me ":
			message = "* %s %s" % (user, message[4:])
		user = user.replace("/", "_")
		self.write(os.path.join("private", user), message, timestamp)
	
	def room(self, room, user, message):
		if message[:4] == "/me":
			message = "* %s %s" % (user, message[4:])
		else:
			message = "[%s] %s" % (user, message)
		room = room.replace("/", "_")
		self.write(os.path.join("room", room), message)

if len(sys.argv) not in (3, 4):
	print "Syntax: mulog.py host:port password [path]"
	print "or:     mulog.py /path/to/socket password [path]"
	sys.exit(-1)

(host, password) = sys.argv[1:3]
if len(sys.argv) == 4:
	path = sys.argv[3]
else:
	path = "~/.museekd/logs"

logger = Logger(path)
driver = Mulog(logger)
driver.connect(host, password, messages.EM_CHAT | messages.EM_PRIVATE)
while 1:
	driver.process()
